// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef StepElement_Array1OfVolumeElementPurposeMember_HeaderFile
#define StepElement_Array1OfVolumeElementPurposeMember_HeaderFile

#include <StepElement_VolumeElementPurposeMember.hxx>
#include <NCollection_Array1.hxx>

typedef NCollection_Array1<Handle(StepElement_VolumeElementPurposeMember)> StepElement_Array1OfVolumeElementPurposeMember;


#endif
