// Created on: 2008-01-21
// Created by: Galina KULIKOVA
// Copyright (c) 2008-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef Interface_VectorOfFileParameter_HeaderFile
#define Interface_VectorOfFileParameter_HeaderFile

#include <MMgt_TShared.hxx>
#include <NCollection_Vector.hxx>
#include <Interface_FileParameter.hxx>

/**
 * Block of comments describing class Interface_VectorOfFileParameter
 */
typedef NCollection_Vector<Interface_FileParameter> Interface_VectorOfFileParameter;

                                         
#endif
